/*
 *  DLC - Utility for dynamic loading of classes
 *  Copyright (C) 1999-2001 Martin Reinecke
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  See the README file for more information.
 */

#include <dlfcn.h>

#include "dlc_server.h"
#include "dlc_maker_base.h"

namespace DLC {

using namespace std;

//explicit
MAKER_BASE::MAKER_BASE (const char *Id)
  : Identity (Id)
  { Server.enter (Identity, this); }

MAKER_BASE::~MAKER_BASE ()
  { Server.remove (Identity); }


SERVER::~SERVER ()
  {
  for (DLLMap::iterator iter = DLLs.begin(); iter != DLLs.end(); ++iter)
    dlclose (iter->second);
  }

void SERVER::load (const string &name)
  {
  if (DLLs.find (name) != DLLs.end())
    throw FILE_ALREADY_LOADED();
  void *tmp = dlopen (name.c_str(), RTLD_NOW);
  if (!tmp)
    throw FILE_OPEN_ERROR();
  DLLs[name] = tmp;
  }

void SERVER::unload (const string &name)
  {
  DLLMap::iterator iter = DLLs.find (name);
  if (iter == DLLs.end())
    throw FILE_NOT_LOADED();
  dlclose (iter->second);
  DLLs.erase(iter);
  }

BASE *SERVER::Get_Instance (const string &name)
  {
  ClassMap::const_iterator iter = Classes.find (name);
  if (iter == Classes.end()) return 0;
  return (iter->second)->Create();
  }

void SERVER::enter (const string &Id, const MAKER_BASE *const ptr)
  {
  if (Classes.find (Id) != Classes.end())
    throw CLASS_ALREADY_LOADED();
  Classes[Id] = ptr;
  }

void SERVER::remove (const string &Id)
  {
  ClassMap::iterator iter = Classes.find (Id);
  if (iter == Classes.end())
    throw CLASS_NOT_LOADED();
  Classes.erase (iter);
  }

SERVER Server;

} // namespace DLC
