/*
 *  DLC - Utility for dynamic loading of classes
 *  Copyright (C) 1999-2001 Martin Reinecke
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  See the README file for more information.
 */

#ifndef DLC_REGISTER_H
#define DLC_REGISTER_H

/**
\file dlc_register.h
Definition of DLC_Register_Macro().
This header file must be included in every file containing a definition
of a dynamically loadable class.
*/

#include "dlc_maker_base.h"

namespace DLC {

template <class T> class MAKER: public MAKER_BASE
  {
  public:
    explicit MAKER (const char *Id) : MAKER_BASE (Id) {}

    virtual BASE *Create() const { return new T; }
  };

} // namespace DLC

/*!
\def DLC_Register_Macro(Cls)
This macro must be "called" for each dynamically loadable class.
It is just a shorthand for convenience.<br>
<tt>DLC_Register_Macro (foo)</tt> stands for<br>
<tt>static DLC::MAKER<foo> DLC_Reg_foo("foo");</tt>
\param Cls The name of the class
\example dlc_test.cxx
*/
#define DLC_Register_Macro(Cls) static DLC::MAKER<Cls> DLC_Reg_##Cls(#Cls);

#endif
